@@ -2,9 +2,10 @@ from enum import Enum
 
 # Model specific simulators mapping:
 # Entries take the form:  full_model_class_path -> (simulator_path, simulator_class_name)
+# TODO: use qualified names
 
 map_model_simulator = {
-    'geckopy.gecko.GeckoModel': ('mewpy.simulation.cobra',    'GeckoSimulation'),
+    'geckopy.gecko.GeckoModel': ('mewpy.simulation.cobra', 'GeckoSimulation'),
     'mewpy.model.gecko.GeckoModel': ('mewpy.simulation.reframed', 'GeckoSimulation'),
     'mewpy.model.smoment.SMomentModel': ('mewpy.simulation.reframed', 'GeckoSimulation')
 }
@@ -12,15 +13,15 @@ map_model_simulator = {
 
 def get_simulator(model, envcond=None, constraints=None, reference=None):
     """
-    Returns a simulator instance for the model
+    Returns a simulator instance for the model.
     The simulator instance is model dependent.
-
-    This function is invoked by a EA optimization problem and by evaluation function instances.
+    Besides able to be used on its own, this function is invoked by EA optimization problems 
+    and by evaluation function instances to perform phenotyoe evaluations of candidate solutions.
 
     
-    :param model : the model
-    :param dic envcond: A dictionary of environmental conditions.
-    :param dic contrainsts: A dictionary of additional persistent constraints.
+    :param model: the model
+    :param dict envcond: A dictionary of environmental conditions.
+    :param dict contrainsts: A dictionary of additional persistent constraints.
     :returns: An instance of Simulator
     """
     instance = None
@@ -50,7 +51,7 @@ def get_simulator(model, envcond=None, constraints=None, reference=None):
             except ImportError:
                 pass
     if not instance:
-        raise ValueError(f"The model [{name}] has no defined simulator.")
+        raise ValueError(f"The model <{name}> has no defined simulator.")
     return instance
 
 
@@ -59,7 +60,7 @@ def get_container(model):
     Returns a container for a given model sharing a common interface.
     A container does not perform any task, it only serves as a basic interface with a phenotype simulator.
 
-    :param model: A metabolic model instance.
+    :param model: An instance of a metabolic model.
     :returns: A container.
      
     """
@@ -83,19 +84,37 @@ def get_container(model):
 
 
 class SimulationMethod(Enum):
-    FBA = 'FBA'
-    pFBA = 'pFBA'
-    MOMA = 'MOMA'
+    
+    FBA   = 'FBA'
+    pFBA  = 'pFBA'
+    MOMA  = 'MOMA'
     lMOMA = 'lMOMA'
-    ROOM = 'ROOM'
-    NONE = 'NONE'
+    ROOM  = 'ROOM'
+    NONE  = 'NONE'
+    
+    def __eq__(self, other):
+        """Overrides equal to enable string name comparison.
+        Allows to seamlessly use: 
+            SimulationMethod.FBA = SimulationMethod.FBA
+            SimulationMethod.FBA = 'FBA'
+        without requiring an additional level of comparison (SimulationMethod.FBA.name = 'FBA')
+        """
+        if isinstance(other,SimulationMethod):
+            return super().__eq__(other)
+        elif isinstance(other,str):
+            return self.name == other
+        else:
+            return False
+
+    def __hash__(self):
+        return hash(self.name)
 
 
 class SStatus(Enum):
     """ Enumeration of possible solution status. """
-    OPTIMAL = 'Optimal'
-    UNKNOWN = 'Unknown'
+    OPTIMAL    = 'Optimal'
+    UNKNOWN    = 'Unknown'
     SUBOPTIMAL = 'Suboptimal'
-    UNBOUNDED = 'Unbounded'
+    UNBOUNDED  = 'Unbounded'
     INFEASIBLE = 'Infeasible'
     INF_OR_UNB = 'Infeasible or Unbounded'
